@@ -51,7 +51,6 @@
 **
 */
 #include "sqliteInt.h"
-#include <math.h>
 
 /*
 ** Conversion types fall into various categories as defined by the
@@ -66,15 +65,14 @@
 #define etDYNSTRING   7 /* Dynamically allocated strings. %z */
 #define etPERCENT     8 /* Percent symbol. %% */
 #define etCHARX       9 /* Characters. %c */
+#define etERROR      10 /* Used to indicate no such conversion type */
 /* The rest are extensions, not normally found in printf() */
-#define etCHARLIT    10 /* Literal characters.  %' */
-#define etSQLESCAPE  11 /* Strings with '\'' doubled.  %q */
-#define etSQLESCAPE2 12 /* Strings with '\'' doubled and enclosed in '',
+#define etCHARLIT    11 /* Literal characters.  %' */
+#define etSQLESCAPE  12 /* Strings with '\'' doubled.  %q */
+#define etSQLESCAPE2 13 /* Strings with '\'' doubled and enclosed in '',
                           NULL pointers replaced by SQL NULL.  %Q */
-#define etTOKEN      13 /* a pointer to a Token structure */
-#define etSRCLIST    14 /* a pointer to a SrcList */
-#define etPOINTER    15 /* The %p conversion */
-#define etSQLESCAPE3 16 /* %w -> Strings with '\"' doubled */
+#define etTOKEN      14 /* a pointer to a Token structure */
+#define etSRCLIST    15 /* a pointer to a SrcList */
 
 
 /*
@@ -91,8 +89,8 @@ typedef struct et_info {   /* Information about each format field */
   etByte base;             /* The base for radix conversion */
   etByte flags;            /* One or more of FLAG_ constants below */
   etByte type;             /* Conversion paradigm */
-  etByte charset;          /* Offset into aDigits[] of the digits string */
-  etByte prefix;           /* Offset into aPrefix[] of the prefix string */
+  char *charset;           /* The character set for conversion */
+  char *prefix;            /* Prefix on non-zero values in alt format */
 } et_info;
 
 /*
@@ -100,48 +98,42 @@ typedef struct et_info {   /* Information about each format field */
 */
 #define FLAG_SIGNED  1     /* True if the value to convert is signed */
 #define FLAG_INTERN  2     /* True if for internal use only */
-#define FLAG_STRING  4     /* Allow infinity precision */
 
 
 /*
 ** The following table is searched linearly, so it is good to put the
 ** most frequently used conversion types first.
 */
-static const char aDigits[] = "0123456789ABCDEF0123456789abcdef";
-static const char aPrefix[] = "-x0\000X0";
-static const et_info fmtinfo[] = {
-  {  'd', 10, 1, etRADIX,      0,  0 },
-  {  's',  0, 4, etSTRING,     0,  0 },
-  {  'g',  0, 1, etGENERIC,    30, 0 },
-  {  'z',  0, 6, etDYNSTRING,  0,  0 },
-  {  'q',  0, 4, etSQLESCAPE,  0,  0 },
-  {  'Q',  0, 4, etSQLESCAPE2, 0,  0 },
-  {  'w',  0, 4, etSQLESCAPE3, 0,  0 },
-  {  'c',  0, 0, etCHARX,      0,  0 },
-  {  'o',  8, 0, etRADIX,      0,  2 },
-  {  'u', 10, 0, etRADIX,      0,  0 },
-  {  'x', 16, 0, etRADIX,      16, 1 },
-  {  'X', 16, 0, etRADIX,      0,  4 },
-#ifndef SQLITE_OMIT_FLOATING_POINT
-  {  'f',  0, 1, etFLOAT,      0,  0 },
-  {  'e',  0, 1, etEXP,        30, 0 },
-  {  'E',  0, 1, etEXP,        14, 0 },
-  {  'G',  0, 1, etGENERIC,    14, 0 },
-#endif
-  {  'i', 10, 1, etRADIX,      0,  0 },
-  {  'n',  0, 0, etSIZE,       0,  0 },
-  {  '%',  0, 0, etPERCENT,    0,  0 },
-  {  'p', 16, 0, etPOINTER,    0,  1 },
-  {  'T',  0, 2, etTOKEN,      0,  0 },
-  {  'S',  0, 2, etSRCLIST,    0,  0 },
+static et_info fmtinfo[] = {
+  {  'd', 10, 1, etRADIX,      "0123456789",       0    },
+  {  's',  0, 0, etSTRING,     0,                  0    },
+  {  'z',  0, 2, etDYNSTRING,  0,                  0    },
+  {  'q',  0, 0, etSQLESCAPE,  0,                  0    },
+  {  'Q',  0, 0, etSQLESCAPE2, 0,                  0    },
+  {  'c',  0, 0, etCHARX,      0,                  0    },
+  {  'o',  8, 0, etRADIX,      "01234567",         "0"  },
+  {  'u', 10, 0, etRADIX,      "0123456789",       0    },
+  {  'x', 16, 0, etRADIX,      "0123456789abcdef", "x0" },
+  {  'X', 16, 0, etRADIX,      "0123456789ABCDEF", "X0" },
+  {  'f',  0, 1, etFLOAT,      0,                  0    },
+  {  'e',  0, 1, etEXP,        "e",                0    },
+  {  'E',  0, 1, etEXP,        "E",                0    },
+  {  'g',  0, 1, etGENERIC,    "e",                0    },
+  {  'G',  0, 1, etGENERIC,    "E",                0    },
+  {  'i', 10, 1, etRADIX,      "0123456789",       0    },
+  {  'n',  0, 0, etSIZE,       0,                  0    },
+  {  '%',  0, 0, etPERCENT,    0,                  0    },
+  {  'p', 10, 0, etRADIX,      "0123456789",       0    },
+  {  'T',  0, 2, etTOKEN,      0,                  0    },
+  {  'S',  0, 2, etSRCLIST,    0,                  0    },
 };
 #define etNINFO  (sizeof(fmtinfo)/sizeof(fmtinfo[0]))
 
 /*
-** If SQLITE_OMIT_FLOATING_POINT is defined, then none of the floating point
+** If NOFLOATINGPOINT is defined, then none of the floating point
 ** conversions will work.
 */
-#ifndef SQLITE_OMIT_FLOATING_POINT
+#ifndef etNOFLOATINGPOINT
 /*
 ** "*val" is a double such that 0.1 <= *val < 10.0
 ** Return the ascii code for the leading digit of *val, then
@@ -165,17 +157,9 @@ static int et_getdigit(LONGDOUBLE_TYPE *val, int *cnt){
   *val = (*val - d)*10.0;
   return digit;
 }
-#endif /* SQLITE_OMIT_FLOATING_POINT */
-
-/*
-** On machines with a small stack size, you can redefine the
-** SQLITE_PRINT_BUF_SIZE to be less than 350.  But beware - for
-** smaller values some %f conversions may go into an infinite loop.
-*/
-#ifndef SQLITE_PRINT_BUF_SIZE
-# define SQLITE_PRINT_BUF_SIZE 350
 #endif
-#define etBUFSIZE SQLITE_PRINT_BUF_SIZE  /* Size of the output buffer */
+
+#define etBUFSIZE 1000  /* Size of the output buffer */
 
 /*
 ** The root program.  All variations call this core.
@@ -222,24 +206,20 @@ static int vxprintf(
   etByte flag_plussign;      /* True if "+" flag is present */
   etByte flag_blanksign;     /* True if " " flag is present */
   etByte flag_alternateform; /* True if "#" flag is present */
-  etByte flag_altform2;      /* True if "!" flag is present */
   etByte flag_zeropad;       /* True if field width constant starts with zero */
   etByte flag_long;          /* True if "l" flag is present */
-  etByte flag_longlong;      /* True if the "ll" flag is present */
-  etByte done;               /* Loop termination flag */
-  sqlite_uint64 longvalue;   /* Value for integer types */
+  unsigned long longvalue;   /* Value for integer types */
   LONGDOUBLE_TYPE realvalue; /* Value for real types */
-  const et_info *infop;      /* Pointer to the appropriate info structure */
+  et_info *infop;            /* Pointer to the appropriate info structure */
   char buf[etBUFSIZE];       /* Conversion buffer */
   char prefix;               /* Prefix character.  "+" or "-" or " " or '\0'. */
   etByte errorflag = 0;      /* True if an error is encountered */
   etByte xtype;              /* Conversion paradigm */
   char *zExtra;              /* Extra memory used for etTCLESCAPE conversions */
-  static const char spaces[] =
-   "                                                                         ";
+  static char spaces[] = "                                                  ";
 #define etSPACESIZE (sizeof(spaces)-1)
-#ifndef SQLITE_OMIT_FLOATING_POINT
-  int  exp, e2;              /* exponent of real numbers */
+#ifndef etNOFLOATINGPOINT
+  int  exp;                  /* exponent of real numbers */
   double rounder;            /* Used for rounding floating point values */
   etByte flag_dp;            /* True if decimal point should be shown */
   etByte flag_rtz;           /* True if trailing zeros should be removed */
@@ -268,19 +248,17 @@ static int vxprintf(
     }
     /* Find out what flags are present */
     flag_leftjustify = flag_plussign = flag_blanksign = 
-     flag_alternateform = flag_altform2 = flag_zeropad = 0;
-    done = 0;
+     flag_alternateform = flag_zeropad = 0;
     do{
       switch( c ){
-        case '-':   flag_leftjustify = 1;     break;
-        case '+':   flag_plussign = 1;        break;
-        case ' ':   flag_blanksign = 1;       break;
-        case '#':   flag_alternateform = 1;   break;
-        case '!':   flag_altform2 = 1;        break;
-        case '0':   flag_zeropad = 1;         break;
-        default:    done = 1;                 break;
+        case '-':   flag_leftjustify = 1;     c = 0;   break;
+        case '+':   flag_plussign = 1;        c = 0;   break;
+        case ' ':   flag_blanksign = 1;       c = 0;   break;
+        case '#':   flag_alternateform = 1;   c = 0;   break;
+        case '0':   flag_zeropad = 1;         c = 0;   break;
+        default:                                       break;
       }
-    }while( !done && (c=(*++fmt))!=0 );
+    }while( c==0 && (c=(*++fmt))!=0 );
     /* Get the field width */
     width = 0;
     if( c=='*' ){
@@ -313,6 +291,8 @@ static int vxprintf(
           c = *++fmt;
         }
       }
+      /* Limit the precision to prevent overflowing buf[] during conversion */
+      if( precision>etBUFSIZE-40 ) precision = etBUFSIZE-40;
     }else{
       precision = -1;
     }
@@ -320,52 +300,33 @@ static int vxprintf(
     if( c=='l' ){
       flag_long = 1;
       c = *++fmt;
-      if( c=='l' ){
-        flag_longlong = 1;
-        c = *++fmt;
-      }else{
-        flag_longlong = 0;
-      }
     }else{
-      flag_long = flag_longlong = 0;
+      flag_long = 0;
     }
     /* Fetch the info entry for the field */
     infop = 0;
+    xtype = etERROR;
     for(idx=0; idx<etNINFO; idx++){
       if( c==fmtinfo[idx].fmttype ){
         infop = &fmtinfo[idx];
         if( useExtended || (infop->flags & FLAG_INTERN)==0 ){
           xtype = infop->type;
-        }else{
-          return -1;
         }
         break;
       }
     }
     zExtra = 0;
-    if( infop==0 ){
-      return -1;
-    }
-
-
-    /* Limit the precision to prevent overflowing buf[] during conversion */
-    if( precision>etBUFSIZE-40 && (infop->flags & FLAG_STRING)==0 ){
-      precision = etBUFSIZE-40;
-    }
 
     /*
     ** At this point, variables are initialized as follows:
     **
     **   flag_alternateform          TRUE if a '#' is present.
-    **   flag_altform2               TRUE if a '!' is present.
     **   flag_plussign               TRUE if a '+' is present.
     **   flag_leftjustify            TRUE if a '-' is present or if the
     **                               field width was negative.
     **   flag_zeropad                TRUE if the width began with 0.
     **   flag_long                   TRUE if the letter 'l' (ell) prefixed
     **                               the conversion character.
-    **   flag_longlong               TRUE if the letter 'll' (ell ell) prefixed
-    **                               the conversion character.
     **   flag_blanksign              TRUE if a ' ' is present.
     **   width                       The specified field width.  This is
     **                               always non-negative.  Zero is the default.
@@ -375,40 +336,34 @@ static int vxprintf(
     **   infop                       Pointer to the appropriate info struct.
     */
     switch( xtype ){
-      case etPOINTER:
-        flag_longlong = sizeof(char*)==sizeof(i64);
-        flag_long = sizeof(char*)==sizeof(long int);
-        /* Fall through into the next case */
       case etRADIX:
-        if( infop->flags & FLAG_SIGNED ){
-          i64 v;
-          if( flag_longlong )   v = va_arg(ap,i64);
-          else if( flag_long )  v = va_arg(ap,long int);
-          else                  v = va_arg(ap,int);
-          if( v<0 ){
-            longvalue = -v;
-            prefix = '-';
-          }else{
-            longvalue = v;
-            if( flag_plussign )        prefix = '+';
-            else if( flag_blanksign )  prefix = ' ';
-            else                       prefix = 0;
-          }
-        }else{
-          if( flag_longlong )   longvalue = va_arg(ap,u64);
-          else if( flag_long )  longvalue = va_arg(ap,unsigned long int);
-          else                  longvalue = va_arg(ap,unsigned int);
-          prefix = 0;
-        }
+        if( flag_long )  longvalue = va_arg(ap,long);
+        else             longvalue = va_arg(ap,int);
+#if 1
+        /* For the format %#x, the value zero is printed "0" not "0x0".
+        ** I think this is stupid. */
         if( longvalue==0 ) flag_alternateform = 0;
+#else
+        /* More sensible: turn off the prefix for octal (to prevent "00"),
+        ** but leave the prefix for hex. */
+        if( longvalue==0 && infop->base==8 ) flag_alternateform = 0;
+#endif
+        if( infop->flags & FLAG_SIGNED ){
+          if( *(long*)&longvalue<0 ){
+            longvalue = -*(long*)&longvalue;
+            prefix = '-';
+          }else if( flag_plussign )  prefix = '+';
+          else if( flag_blanksign )  prefix = ' ';
+          else                       prefix = 0;
+        }else                        prefix = 0;
         if( flag_zeropad && precision<width-(prefix!=0) ){
           precision = width-(prefix!=0);
         }
         bufpt = &buf[etBUFSIZE-1];
         {
-          register const char *cset;      /* Use registers for speed */
+          register char *cset;      /* Use registers for speed */
           register int base;
-          cset = &aDigits[infop->charset];
+          cset = infop->charset;
           base = infop->base;
           do{                                           /* Convert to ascii */
             *(--bufpt) = cset[longvalue%base];
@@ -421,11 +376,10 @@ static int vxprintf(
         }
         if( prefix ) *(--bufpt) = prefix;               /* Add sign */
         if( flag_alternateform && infop->prefix ){      /* Add "0" or "0x" */
-          const char *pre;
-          char x;
-          pre = &aPrefix[infop->prefix];
+          char *pre, x;
+          pre = infop->prefix;
           if( *bufpt!=pre[0] ){
-            for(; (x=(*pre))!=0; pre++) *(--bufpt) = x;
+            for(pre=infop->prefix; (x=(*pre))!=0; pre++) *(--bufpt) = x;
           }
         }
         length = &buf[etBUFSIZE-1]-bufpt;
@@ -434,9 +388,9 @@ static int vxprintf(
       case etEXP:
       case etGENERIC:
         realvalue = va_arg(ap,double);
-#ifndef SQLITE_OMIT_FLOATING_POINT
+#ifndef etNOFLOATINGPOINT
         if( precision<0 ) precision = 6;         /* Set default precision */
-        if( precision>etBUFSIZE/2-10 ) precision = etBUFSIZE/2-10;
+        if( precision>etBUFSIZE-10 ) precision = etBUFSIZE-10;
         if( realvalue<0.0 ){
           realvalue = -realvalue;
           prefix = '-';
@@ -445,37 +399,26 @@ static int vxprintf(
           else if( flag_blanksign )    prefix = ' ';
           else                         prefix = 0;
         }
-        if( xtype==etGENERIC && precision>0 ) precision--;
+        if( infop->type==etGENERIC && precision>0 ) precision--;
+        rounder = 0.0;
 #if 0
         /* Rounding works like BSD when the constant 0.4999 is used.  Wierd! */
         for(idx=precision, rounder=0.4999; idx>0; idx--, rounder*=0.1);
 #else
         /* It makes more sense to use 0.5 */
-        for(idx=precision, rounder=0.5; idx>0; idx--, rounder*=0.1){}
+        for(idx=precision, rounder=0.5; idx>0; idx--, rounder*=0.1);
 #endif
-        if( xtype==etFLOAT ) realvalue += rounder;
+        if( infop->type==etFLOAT ) realvalue += rounder;
         /* Normalize realvalue to within 10.0 > realvalue >= 1.0 */
         exp = 0;
-        if( sqlite3_isnan(realvalue) ){
-          bufpt = "NaN";
-          length = 3;
-          break;
-        }
         if( realvalue>0.0 ){
-          while( realvalue>=1e32 && exp<=350 ){ realvalue *= 1e-32; exp+=32; }
           while( realvalue>=1e8 && exp<=350 ){ realvalue *= 1e-8; exp+=8; }
           while( realvalue>=10.0 && exp<=350 ){ realvalue *= 0.1; exp++; }
           while( realvalue<1e-8 && exp>=-350 ){ realvalue *= 1e8; exp-=8; }
           while( realvalue<1.0 && exp>=-350 ){ realvalue *= 10.0; exp--; }
           if( exp>350 || exp<-350 ){
-            if( prefix=='-' ){
-              bufpt = "-Inf";
-            }else if( prefix=='+' ){
-              bufpt = "+Inf";
-            }else{
-              bufpt = "Inf";
-            }
-            length = strlen(bufpt);
+            bufpt = "NaN";
+            length = 3;
             break;
           }
         }
@@ -500,67 +443,51 @@ static int vxprintf(
         }else{
           flag_rtz = 0;
         }
-        if( xtype==etEXP ){
-          e2 = 0;
-        }else{
-          e2 = exp;
-        }
+        /*
+        ** The "exp+precision" test causes output to be of type etEXP if
+        ** the precision is too large to fit in buf[].
+        */
         nsd = 0;
-        flag_dp = (precision>0) | flag_alternateform | flag_altform2;
-        /* The sign in front of the number */
-        if( prefix ){
-          *(bufpt++) = prefix;
-        }
-        /* Digits prior to the decimal point */
-        if( e2<0 ){
-          *(bufpt++) = '0';
-        }else{
-          for(; e2>=0; e2--){
-            *(bufpt++) = et_getdigit(&realvalue,&nsd);
+        if( xtype==etFLOAT && exp+precision<etBUFSIZE-30 ){
+          flag_dp = (precision>0 || flag_alternateform);
+          if( prefix ) *(bufpt++) = prefix;         /* Sign */
+          if( exp<0 )  *(bufpt++) = '0';            /* Digits before "." */
+          else for(; exp>=0; exp--) *(bufpt++) = et_getdigit(&realvalue,&nsd);
+          if( flag_dp ) *(bufpt++) = '.';           /* The decimal point */
+          for(exp++; exp<0 && precision>0; precision--, exp++){
+            *(bufpt++) = '0';
           }
-        }
-        /* The decimal point */
-        if( flag_dp ){
-          *(bufpt++) = '.';
-        }
-        /* "0" digits after the decimal point but before the first
-        ** significant digit of the number */
-        for(e2++; e2<0 && precision>0; precision--, e2++){
-          *(bufpt++) = '0';
-        }
-        /* Significant digits after the decimal point */
-        while( (precision--)>0 ){
-          *(bufpt++) = et_getdigit(&realvalue,&nsd);
-        }
-        /* Remove trailing zeros and the "." if no digits follow the "." */
-        if( flag_rtz && flag_dp ){
-          while( bufpt[-1]=='0' ) *(--bufpt) = 0;
-          assert( bufpt>buf );
-          if( bufpt[-1]=='.' ){
-            if( flag_altform2 ){
-              *(bufpt++) = '0';
-            }else{
-              *(--bufpt) = 0;
+          while( (precision--)>0 ) *(bufpt++) = et_getdigit(&realvalue,&nsd);
+          *(bufpt--) = 0;                           /* Null terminate */
+          if( flag_rtz && flag_dp ){     /* Remove trailing zeros and "." */
+            while( bufpt>=buf && *bufpt=='0' ) *(bufpt--) = 0;
+            if( bufpt>=buf && *bufpt=='.' ) *(bufpt--) = 0;
+          }
+          bufpt++;                            /* point to next free slot */
+        }else{    /* etEXP or etGENERIC */
+          flag_dp = (precision>0 || flag_alternateform);
+          if( prefix ) *(bufpt++) = prefix;   /* Sign */
+          *(bufpt++) = et_getdigit(&realvalue,&nsd);  /* First digit */
+          if( flag_dp ) *(bufpt++) = '.';     /* Decimal point */
+          while( (precision--)>0 ) *(bufpt++) = et_getdigit(&realvalue,&nsd);
+          bufpt--;                            /* point to last digit */
+          if( flag_rtz && flag_dp ){          /* Remove tail zeros */
+            while( bufpt>=buf && *bufpt=='0' ) *(bufpt--) = 0;
+            if( bufpt>=buf && *bufpt=='.' ) *(bufpt--) = 0;
+          }
+          bufpt++;                            /* point to next free slot */
+          if( exp || flag_exp ){
+            *(bufpt++) = infop->charset[0];
+            if( exp<0 ){ *(bufpt++) = '-'; exp = -exp; } /* sign of exp */
+            else       { *(bufpt++) = '+'; }
+            if( exp>=100 ){
+              *(bufpt++) = (exp/100)+'0';                /* 100's digit */
+              exp %= 100;
             }
+            *(bufpt++) = exp/10+'0';                     /* 10's digit */
+            *(bufpt++) = exp%10+'0';                     /* 1's digit */
           }
         }
-        /* Add the "eNNN" suffix */
-        if( flag_exp || (xtype==etEXP && exp) ){
-          *(bufpt++) = aDigits[infop->charset];
-          if( exp<0 ){
-            *(bufpt++) = '-'; exp = -exp;
-          }else{
-            *(bufpt++) = '+';
-          }
-          if( exp>=100 ){
-            *(bufpt++) = (exp/100)+'0';                /* 100's digit */
-            exp %= 100;
-          }
-          *(bufpt++) = exp/10+'0';                     /* 10's digit */
-          *(bufpt++) = exp%10+'0';                     /* 1's digit */
-        }
-        *bufpt = 0;
-
         /* The converted number is in buf[] and zero terminated. Output it.
         ** Note that the number is in the usual order, not reversed as with
         ** integer conversions. */
@@ -614,42 +541,36 @@ static int vxprintf(
         break;
       case etSQLESCAPE:
       case etSQLESCAPE2:
-      case etSQLESCAPE3: {
-        int i, j, n, ch, isnull;
-        int needQuote;
-        char q = ((xtype==etSQLESCAPE3)?'"':'\'');   /* Quote character */
-        char *escarg = va_arg(ap,char*);
-        isnull = escarg==0;
-        if( isnull ) escarg = (xtype==etSQLESCAPE2 ? "NULL" : "(NULL)");
-        for(i=n=0; (ch=escarg[i])!=0; i++){
-          if( ch==q )  n++;
+        {
+          int i, j, n, c, isnull;
+          char *arg = va_arg(ap,char*);
+          isnull = arg==0;
+          if( isnull ) arg = (xtype==etSQLESCAPE2 ? "NULL" : "(NULL)");
+          for(i=n=0; (c=arg[i])!=0; i++){
+            if( c=='\'' )  n++;
+          }
+          n += i + 1 + ((!isnull && xtype==etSQLESCAPE2) ? 2 : 0);
+          if( n>etBUFSIZE ){
+            bufpt = zExtra = sqliteMalloc( n );
+            if( bufpt==0 ) return -1;
+          }else{
+            bufpt = buf;
+          }
+          j = 0;
+          if( !isnull && xtype==etSQLESCAPE2 ) bufpt[j++] = '\'';
+          for(i=0; (c=arg[i])!=0; i++){
+            bufpt[j++] = c;
+            if( c=='\'' ) bufpt[j++] = c;
+          }
+          if( !isnull && xtype==etSQLESCAPE2 ) bufpt[j++] = '\'';
+          bufpt[j] = 0;
+          length = j;
+          if( precision>=0 && precision<length ) length = precision;
         }
-        needQuote = !isnull && xtype==etSQLESCAPE2;
-        n += i + 1 + needQuote*2;
-        if( n>etBUFSIZE ){
-          bufpt = zExtra = sqliteMalloc( n );
-          if( bufpt==0 ) return -1;
-        }else{
-          bufpt = buf;
-        }
-        j = 0;
-        if( needQuote ) bufpt[j++] = q;
-        for(i=0; (ch=escarg[i])!=0; i++){
-          bufpt[j++] = ch;
-          if( ch==q ) bufpt[j++] = ch;
-        }
-        if( needQuote ) bufpt[j++] = q;
-        bufpt[j] = 0;
-        length = j;
-        /* The precision is ignored on %q and %Q */
-        /* if( precision>=0 && precision<length ) length = precision; */
         break;
-      }
       case etTOKEN: {
         Token *pToken = va_arg(ap, Token*);
-        if( pToken && pToken->z ){
-          (*func)(arg, (char*)pToken->z, pToken->n);
-        }
+        (*func)(arg, pToken->z, pToken->n);
         length = width = 0;
         break;
       }
@@ -666,6 +587,15 @@ static int vxprintf(
         length = width = 0;
         break;
       }
+      case etERROR:
+        buf[0] = '%';
+        buf[1] = c;
+        errorflag = 0;
+        idx = 1+(c!=0);
+        (*func)(arg,"%",idx);
+        count += idx;
+        if( c==0 ) fmt--;
+        break;
     }/* End switch over the format type */
     /*
     ** The text of the conversion is pointed to by "bufpt" and is
@@ -733,22 +663,15 @@ static void mout(void *arg, const char *zNewText, int nNewChar){
     if( pM->xRealloc==0 ){
       nNewChar =  pM->nAlloc - pM->nChar - 1;
     }else{
-      int nAlloc = pM->nChar + nNewChar*2 + 1;
+      pM->nAlloc = pM->nChar + nNewChar*2 + 1;
       if( pM->zText==pM->zBase ){
-        pM->zText = pM->xRealloc(0, nAlloc);
+        pM->zText = pM->xRealloc(0, pM->nAlloc);
         if( pM->zText && pM->nChar ){
           memcpy(pM->zText, pM->zBase, pM->nChar);
         }
       }else{
-        char *zNew;
-        zNew = pM->xRealloc(pM->zText, nAlloc);
-        if( zNew ){
-          pM->zText = zNew;
-        }else{
-          return;
-        }
+        pM->zText = pM->xRealloc(pM->zText, pM->nAlloc);
       }
-      pM->nAlloc = nAlloc;
     }
   }
   if( pM->zText ){
@@ -781,14 +704,9 @@ static char *base_vprintf(
   if( xRealloc ){
     if( sM.zText==sM.zBase ){
       sM.zText = xRealloc(0, sM.nChar+1);
-      if( sM.zText ){
-        memcpy(sM.zText, sM.zBase, sM.nChar+1);
-      }
+      memcpy(sM.zText, sM.zBase, sM.nChar+1);
     }else if( sM.nAlloc>sM.nChar+10 ){
-      char *zNew = xRealloc(sM.zText, sM.nChar+1);
-      if( zNew ){
-        sM.zText = zNew;
-      }
+      sM.zText = xRealloc(sM.zText, sM.nChar+1);
     }
   }
   return sM.zText;
@@ -805,8 +723,8 @@ static void *printf_realloc(void *old, int size){
 ** Print into memory obtained from sqliteMalloc().  Use the internal
 ** %-conversion extensions.
 */
-char *sqlite3VMPrintf(const char *zFormat, va_list ap){
-  char zBase[SQLITE_PRINT_BUF_SIZE];
+char *sqliteVMPrintf(const char *zFormat, va_list ap){
+  char zBase[1000];
   return base_vprintf(printf_realloc, 1, zBase, sizeof(zBase), zFormat, ap);
 }
 
@@ -814,10 +732,10 @@ char *sqlite3VMPrintf(const char *zFormat, va_list ap){
 ** Print into memory obtained from sqliteMalloc().  Use the internal
 ** %-conversion extensions.
 */
-char *sqlite3MPrintf(const char *zFormat, ...){
+char *sqliteMPrintf(const char *zFormat, ...){
   va_list ap;
   char *z;
-  char zBase[SQLITE_PRINT_BUF_SIZE];
+  char zBase[1000];
   va_start(ap, zFormat);
   z = base_vprintf(printf_realloc, 1, zBase, sizeof(zBase), zFormat, ap);
   va_end(ap);
@@ -825,61 +743,116 @@ char *sqlite3MPrintf(const char *zFormat, ...){
 }
 
 /*
-** Print into memory obtained from sqlite3_malloc().  Omit the internal
-** %-conversion extensions.
+** Print into memory obtained from malloc().  Do not use the internal
+** %-conversion extensions.  This routine is for use by external users.
 */
-char *sqlite3_vmprintf(const char *zFormat, va_list ap){
-  char zBase[SQLITE_PRINT_BUF_SIZE];
-  return base_vprintf(sqlite3_realloc, 0, zBase, sizeof(zBase), zFormat, ap);
-}
-
-/*
-** Print into memory obtained from sqlite3_malloc()().  Omit the internal
-** %-conversion extensions.
-*/
-char *sqlite3_mprintf(const char *zFormat, ...){
+char *sqlite_mprintf(const char *zFormat, ...){
   va_list ap;
   char *z;
-  va_start(ap, zFormat);
-  z = sqlite3_vmprintf(zFormat, ap);
+  char zBuf[200];
+
+  va_start(ap,zFormat);
+  z = base_vprintf((void*(*)(void*,int))realloc, 0, 
+                   zBuf, sizeof(zBuf), zFormat, ap);
   va_end(ap);
   return z;
 }
 
+/* This is the varargs version of sqlite_mprintf.  
+*/
+char *sqlite_vmprintf(const char *zFormat, va_list ap){
+  char zBuf[200];
+  return base_vprintf((void*(*)(void*,int))realloc, 0,
+                      zBuf, sizeof(zBuf), zFormat, ap);
+}
+
 /*
-** sqlite3_snprintf() works like snprintf() except that it ignores the
+** sqlite_snprintf() works like snprintf() except that it ignores the
 ** current locale settings.  This is important for SQLite because we
 ** are not able to use a "," as the decimal point in place of "." as
 ** specified by some locales.
 */
-char *sqlite3_snprintf(int n, char *zBuf, const char *zFormat, ...){
+char *sqlite_snprintf(int n, char *zBuf, const char *zFormat, ...){
   char *z;
   va_list ap;
 
-  if( n<=0 ){
-    return zBuf;
-  }
-  zBuf[0] = 0;
   va_start(ap,zFormat);
   z = base_vprintf(0, 0, zBuf, n, zFormat, ap);
   va_end(ap);
   return z;
 }
 
-#if defined(SQLITE_TEST) || defined(SQLITE_DEBUG) || defined(SQLITE_MEMDEBUG)
 /*
-** A version of printf() that understands %lld.  Used for debugging.
-** The printf() built into some versions of windows does not understand %lld
-** and segfaults if you give it a long long int.
+** The following four routines implement the varargs versions of the
+** sqlite_exec() and sqlite_get_table() interfaces.  See the sqlite.h
+** header files for a more detailed description of how these interfaces
+** work.
+**
+** These routines are all just simple wrappers.
 */
-void sqlite3DebugPrintf(const char *zFormat, ...){
-  extern int getpid(void);
+int sqlite_exec_printf(
+  sqlite *db,                   /* An open database */
+  const char *sqlFormat,        /* printf-style format string for the SQL */
+  sqlite_callback xCallback,    /* Callback function */
+  void *pArg,                   /* 1st argument to callback function */
+  char **errmsg,                /* Error msg written here */
+  ...                           /* Arguments to the format string. */
+){
   va_list ap;
-  char zBuf[500];
-  va_start(ap, zFormat);
-  base_vprintf(0, 0, zBuf, sizeof(zBuf), zFormat, ap);
+  int rc;
+
+  va_start(ap, errmsg);
+  rc = sqlite_exec_vprintf(db, sqlFormat, xCallback, pArg, errmsg, ap);
   va_end(ap);
-  fprintf(stdout,"%s", zBuf);
-  fflush(stdout);
+  return rc;
+}
+int sqlite_exec_vprintf(
+  sqlite *db,                   /* An open database */
+  const char *sqlFormat,        /* printf-style format string for the SQL */
+  sqlite_callback xCallback,    /* Callback function */
+  void *pArg,                   /* 1st argument to callback function */
+  char **errmsg,                /* Error msg written here */
+  va_list ap                    /* Arguments to the format string. */
+){
+  char *zSql;
+  int rc;
+
+  zSql = sqlite_vmprintf(sqlFormat, ap);
+  rc = sqlite_exec(db, zSql, xCallback, pArg, errmsg);
+  free(zSql);
+  return rc;
+}
+int sqlite_get_table_printf(
+  sqlite *db,            /* An open database */
+  const char *sqlFormat, /* printf-style format string for the SQL */
+  char ***resultp,       /* Result written to a char *[]  that this points to */
+  int *nrow,             /* Number of result rows written here */
+  int *ncol,             /* Number of result columns written here */
+  char **errmsg,         /* Error msg written here */
+  ...                    /* Arguments to the format string */
+){
+  va_list ap;
+  int rc;
+
+  va_start(ap, errmsg);
+  rc = sqlite_get_table_vprintf(db, sqlFormat, resultp, nrow, ncol, errmsg, ap);
+  va_end(ap);
+  return rc;
+}
+int sqlite_get_table_vprintf(
+  sqlite *db,            /* An open database */
+  const char *sqlFormat, /* printf-style format string for the SQL */
+  char ***resultp,       /* Result written to a char *[]  that this points to */
+  int *nrow,             /* Number of result rows written here */
+  int *ncolumn,          /* Number of result columns written here */
+  char **errmsg,         /* Error msg written here */
+  va_list ap             /* Arguments to the format string */
+){
+  char *zSql;
+  int rc;
+
+  zSql = sqlite_vmprintf(sqlFormat, ap);
+  rc = sqlite_get_table(db, zSql, resultp, nrow, ncolumn, errmsg);
+  free(zSql);
+  return rc;
 }
-#endif
